<?php

/**
 * This code was generated by
 * \ / _    _  _|   _  _
 * | (_)\/(_)(_|\/| |(/_  v1.0.0
 * /       /
 */

namespace Twilio\Rest\Supersim\V1;

use Twilio\Options;
use Twilio\Values;

/**
 * PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
 */
abstract class SimOptions {
    /**
     * @param string $uniqueName An application-defined string that uniquely
     *                           identifies the resource
     * @param string $status The new status of the Super SIM
     * @param string $fleet The SID or unique name of the Fleet to which the SIM
     *                      resource should be assigned
     * @return UpdateSimOptions Options builder
     */
    public static function update(string $uniqueName = Values::NONE, string $status = Values::NONE, string $fleet = Values::NONE): UpdateSimOptions {
        return new UpdateSimOptions($uniqueName, $status, $fleet);
    }

    /**
     * @param string $status The status of the Sim resources to read
     * @param string $fleet The SID or unique name of the Fleet to which a list of
     *                      Sims are assigned
     * @param string $iccid The ICCID associated with a Super SIM to filter the
     *                      list by
     * @return ReadSimOptions Options builder
     */
    public static function read(string $status = Values::NONE, string $fleet = Values::NONE, string $iccid = Values::NONE): ReadSimOptions {
        return new ReadSimOptions($status, $fleet, $iccid);
    }
}

class UpdateSimOptions extends Options {
    /**
     * @param string $uniqueName An application-defined string that uniquely
     *                           identifies the resource
     * @param string $status The new status of the Super SIM
     * @param string $fleet The SID or unique name of the Fleet to which the SIM
     *                      resource should be assigned
     */
    public function __construct(string $uniqueName = Values::NONE, string $status = Values::NONE, string $fleet = Values::NONE) {
        $this->options['uniqueName'] = $uniqueName;
        $this->options['status'] = $status;
        $this->options['fleet'] = $fleet;
    }

    /**
     * An application-defined string that uniquely identifies the resource. It can be used in place of the resource's `sid` in the URL to address the resource.
     *
     * @param string $uniqueName An application-defined string that uniquely
     *                           identifies the resource
     * @return $this Fluent Builder
     */
    public function setUniqueName(string $uniqueName): self {
        $this->options['uniqueName'] = $uniqueName;
        return $this;
    }

    /**
     * The new status of the resource. Can be: `active` or `inactive`. See the [Super SIM Status Values](https://www.twilio.com/docs/iot/supersim/api/sim-resource#status-values) for more info.
     *
     * @param string $status The new status of the Super SIM
     * @return $this Fluent Builder
     */
    public function setStatus(string $status): self {
        $this->options['status'] = $status;
        return $this;
    }

    /**
     * The SID or unique name of the Fleet to which the SIM resource should be assigned.
     *
     * @param string $fleet The SID or unique name of the Fleet to which the SIM
     *                      resource should be assigned
     * @return $this Fluent Builder
     */
    public function setFleet(string $fleet): self {
        $this->options['fleet'] = $fleet;
        return $this;
    }

    /**
     * Provide a friendly representation
     *
     * @return string Machine friendly representation
     */
    public function __toString(): string {
        $options = \http_build_query(Values::of($this->options), '', ' ');
        return '[Twilio.Supersim.V1.UpdateSimOptions ' . $options . ']';
    }
}

class ReadSimOptions extends Options {
    /**
     * @param string $status The status of the Sim resources to read
     * @param string $fleet The SID or unique name of the Fleet to which a list of
     *                      Sims are assigned
     * @param string $iccid The ICCID associated with a Super SIM to filter the
     *                      list by
     */
    public function __construct(string $status = Values::NONE, string $fleet = Values::NONE, string $iccid = Values::NONE) {
        $this->options['status'] = $status;
        $this->options['fleet'] = $fleet;
        $this->options['iccid'] = $iccid;
    }

    /**
     * The status of the Sim resources to read. Can be `new`, `active`, `inactive`, or `scheduled`.
     *
     * @param string $status The status of the Sim resources to read
     * @return $this Fluent Builder
     */
    public function setStatus(string $status): self {
        $this->options['status'] = $status;
        return $this;
    }

    /**
     * The SID or unique name of the Fleet to which a list of Sims are assigned.
     *
     * @param string $fleet The SID or unique name of the Fleet to which a list of
     *                      Sims are assigned
     * @return $this Fluent Builder
     */
    public function setFleet(string $fleet): self {
        $this->options['fleet'] = $fleet;
        return $this;
    }

    /**
     * The [ICCID](https://en.wikipedia.org/wiki/Subscriber_identity_module#ICCID) associated with a Super SIM to filter the list by. Passing this parameter will always return a list containing zero or one SIMs.
     *
     * @param string $iccid The ICCID associated with a Super SIM to filter the
     *                      list by
     * @return $this Fluent Builder
     */
    public function setIccid(string $iccid): self {
        $this->options['iccid'] = $iccid;
        return $this;
    }

    /**
     * Provide a friendly representation
     *
     * @return string Machine friendly representation
     */
    public function __toString(): string {
        $options = \http_build_query(Values::of($this->options), '', ' ');
        return '[Twilio.Supersim.V1.ReadSimOptions ' . $options . ']';
    }
}